"""contain ping-test for re6set net"""
import unittest
import time
import re6st_wrap
import subprocess
import sys
import os

from pathlib2 import Path
from network_build import *

PING_PATH = Path(__file__).parent.resolve() / "ping.py"

def wait_stable(nodes, timeout=180):
    sys.stderr.write("wait stalbe\n")
    now = time.time()
    # wait all the nodes can connect to each other
    ips = {node.ip6:node.name for node in nodes}
    for node in nodes:
        sub_ips = set(ips) - {node.ip6}
        node.ping_proc = node.node.Popen(
            "python {} --retry -a {} ".format(PING_PATH, ' '.join(sub_ips)),shell=True)
    unfinished = list(nodes)
    # check all the node network can ping each other
    while len(unfinished):
        for i in range(len(unfinished)-1,-1,-1):
            node = unfinished[i]
            if node.ping_proc.poll() is not None:
                sys.stderr.write("{}'s network is stable\n".format(node.name))
                unfinished.pop(i)
                node.ping_proc.wait()
        time.sleep(0.5)
        if time.time() - now > timeout:
            raise Exception("wait network stable timeout")

def ping_test(nodes):
    """test Re6st node connecty """
    ips = {node.ip6:node.name for node in nodes}
    sys.stderr.write("start ping test\n")
    failed = False
    for node in nodes:
        sub_ips = set(ips) - {node.ip6}
        node.ping_proc = node.node.Popen(
            "python {} -a {} ".format(PING_PATH, ' '.join(sub_ips)),
            shell=True, stdout=subprocess.PIPE)
        out, _ = node.ping_proc.communicate()
        unreached = [ ips[addr] for addr in out.split()]
        if unreached:
            sys.stderr.write("{} can't ping to {}\n".format(node.name, ",".join(unreached)))
            failed = True
    sys.stderr.write("ping test finish\n")
    return failed


@unittest.skipIf(os.geteuid() != 0, "require root")
class TestPing(unittest.TestCase):
    """" ping-test case"""

    @classmethod
    def setUpClass(cls):
        """set up for all case"""
        # re6st_wrap.WORK_DIR =  xxx # chage the work dir
        re6st_wrap.initial()

    def tearDown(self):
        # terminate all the sub process
        return
        p = subprocess.Popen("ps -aux | grep re6st | grep -v grep", shell=True, stdout =subprocess.PIPE)
        out, _ = p.communicate()
        out = str(out)
        out = out.split('\n')
        out.pop()
        for proc in out:
            proc = proc.split()[1]
            subprocess.call(["kill", "-15", proc])

    # def test_sample(self):
    #     """create a network demo, test the connectivity by ping
    #     wait the network stable then ping 3 times
    #     """
    #     nm = network_direct()
    #     net = nm.registrys
    #     nodes = []
    #     registrys = []
    #     for registry in net:
    #         reg = re6st_wrap.Re6stRegistry(registry, "2001:db8:42::", recreate=False)
    #         reg_node = re6st_wrap.Re6stNode(registry, reg, name=reg.name)
    #         registrys.append(registry)
    #         reg_node.run("--gateway", "--disable-proto", "none", "--ip", registry.ip)
    #         nodes.append(reg_node)
    #         for m in net[registry]:
    #             node = re6st_wrap.Re6stNode(m, reg)
    #             node.run("-i" + m.iface.name)
    #             nodes.append(node)

    #     wait_stable(nodes)
    #     for i in range(3):
    #         time.sleep(20)
    #         self.assertFalse(ping_test(nodes), "N.{} ping test failed".format(i))

    def test_demo(self):
        """create a network demo, test the connectivity by ping
        wait the network stable then ping 3 times
        """
        nm = network_demo()
        net = nm.registrys
        nodes = []
        registrys = []
        for registry in net:
            reg = re6st_wrap.Re6stRegistry(registry, "2001:db8:42::", recreate=False)
            reg_node = re6st_wrap.Re6stNode(registry, reg, name=reg.name)
            registrys.append(registry)
            reg_node.run("--gateway", "--disable-proto", "none", "--ip", registry.ip)
            nodes.append(reg_node)
            for m in net[registry]:
                node = re6st_wrap.Re6stNode(m, reg)
                node.run("-i" + m.iface.name)
                nodes.append(node)

        wait_stable(nodes)
        for i in range(3):
            time.sleep(30)
            self.assertFalse(ping_test(nodes), "N.{} ping test failed".format(i))


if __name__ == "__main__":
    unittest.main()





